#ifndef HEJRIVET_H
#define HEJRIVET_H

#include "HepMCInterface.h"
#include "HepMC/GenEvent.h"
#include "HepMC/IO_GenEvent.h"
#include <set>
#include <vector>
#include <string>
#include "Rivet/AnalysisHandler.hh"
#include <fstream>
#include <iostream>
#include "HepMC/PythiaWrapper.h"
#include "HepMC/IO_HEPEVT.h"

/**
 * Simplified interface to the Rivet program. Remember to link with
 * pythia and -lhepmcinterface -lHepMC -lRivet
 *
 * Usage: (1) Create an object giving a filename as argument. (2)
 * Repeatedly specify (the name of an) analysis with the addAnalysis()
 * function. (3) initialize the underlying Rivet object with the
 * init() function. (4) Analyze an event with the operator() function,
 * giving the event number, the weight, and the current evaluation of
 * the cross section. (5) Dump the histograms to a file with the
 * done() function.
 */
class HejRivet {

public:

  /**
   * The constructor needs to have the name of the file where the
   * histograms are dumped. (.aida suffix wil be added).
   */
  HejRivet(std::string fname)
    : filename(fname), rivet(0) {}

  /**
   * The destructor will write the histogram file if this has not
   * already been done.
   */
  ~HejRivet() {
    done();
  }

  /**
   * Add the name of an analysis to be performed. Will silently fail
   * if rivet has already been initialized.
   */
  void addAnalysis(std::string ana) {
    analyses.insert(ana);
  }

  /**
   * Initialize Rivet. Will do nothing if Rivet was already initialized
   */
  void init() {
    if ( rivet ) return;
    rivet = new Rivet::AnalysisHandler(filename);
    std::vector<std::string> anals(analyses.begin(), analyses.end());
    rivet->addAnalyses(anals);
    rivet->init();
  }

  /**
   * Analyze the given event. Will do nothing if Rivet has not been
   * intialized.
   */
  void operator()(int eventno, double weight, double xsec) {
    if ( !rivet ) return;
    call_pyhepc( 1 );
    HepMC::GenEvent evt;
    evt.use_units(HepMC::Units::GEV, HepMC::Units::MM);
    hepevtio.fill_next_event(&evt);
    evt.set_event_number(eventno);
    HepMC::GenCrossSection xs;
    xs.set_cross_section(xsec);
    evt.set_cross_section( xs );
    evt.weights().push_back(weight);
    rivet->analyze(evt);
  }

  /**
   * Writes histograms to file and deletes the Rivet object. Does
   * nothing if Rivet was not initialized.
   */
  void done() {
    if ( !rivet ) return;
    rivet->finalize();
    rivet->commitData();
    delete rivet;
    rivet = 0;
  }

private:

  /**
   * The name of the file where the histograms are dumped.
   */
  std::string filename;

  /**
   * The names of the selected analyses.
   */
  std::set<string> analyses;

  /**
   * The Rivet object.
   */
  Rivet::AnalysisHandler * rivet;

  /**
   * The HepMC converter
   */
  HepMC::IO_HEPEVT hepevtio;

};

#endif
